@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_TRANSFORMED_CONTAINER_H
-#define POLYMAKE_TRANSFORMED_CONTAINER_H
+#pragma once
 
 #include "polymake/internal/sparse.h"
 #include "polymake/internal/modified_containers.h"
@@ -737,7 +736,6 @@ namespace polymake {
    using pm::repeat;
 }
 
-#endif // POLYMAKE_TRANSFORMED_CONTAINER_H
 
 // Local Variables:
 // mode:C++
