@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -24,6 +24,7 @@
 #include "polymake/linalg.h"
 #include "polymake/IncidenceMatrix.h"
 #include "polymake/FacetList.h"
+#include "polymake/polytope/minkowski_sum_fukuda.h"
 
 namespace polymake { namespace fan {
 
@@ -48,9 +49,9 @@ Matrix<E> express_in_basis(const Matrix<E>& A, const Matrix<E>& B)
 }  // end anonymous namespace
 
 template <typename E>
-perl::Object tiling_quotient(perl::Object P, perl::Object Q)
+BigObject tiling_quotient(BigObject P, BigObject Q)
 {
-   const int
+   const Int
       d = P.give("COMBINATORIAL_DIM"),
       e = Q.give("COMBINATORIAL_DIM");
    if (d!=e) throw std::runtime_error("The dimensions of P and Q must be equal");
@@ -74,16 +75,14 @@ perl::Object tiling_quotient(perl::Object P, perl::Object Q)
    VQL -= repeat_row(barycenter, VQL.rows());
 
    // Take the Minkowski sum of the transformed P and Q 
-   perl::ObjectType Polytope("Polytope", mlist<E>());
-   perl::Object summand1(Polytope);
-   perl::Object summand2(Polytope);
-   summand1.take("VERTICES") << (ones_vector<E>() | VPL);
-   summand2.take("VERTICES") << (ones_vector<E>() | VQL);
-   const Matrix<E> MV = call_function("polytope::minkowski_sum_vertices_fukuda", mlist<E>(), summand1, summand2);
+   Array<BigObject> summands(2);
+   BigObjectType Polytope("Polytope", mlist<E>());
+   summands[0] = BigObject(Polytope, "VERTICES", ones_vector<E>() | VPL);
+   summands[1] = BigObject(Polytope, "VERTICES", ones_vector<E>() | VQL);
+   const Matrix<E> MV = polytope::minkowski_sum_vertices_fukuda<E>(summands).first;
 
    // Find the interior and boundary lattice points of the Minkowski sum
-   perl::Object M("Polytope", mlist<E>());
-   M.take("VERTICES") << MV;
+   BigObject M("Polytope", mlist<E>(), "VERTICES", MV);
    const Matrix<E> 
       ILP = M.give("INTERIOR_LATTICE_POINTS"),
       BLP = M.give("BOUNDARY_LATTICE_POINTS"),
@@ -93,11 +92,11 @@ perl::Object tiling_quotient(perl::Object P, perl::Object Q)
    // get the facets of the transformed polytopes.
    // of course, we could calculate these directly...
    const Matrix<E> 
-      FP = summand1.give("FACETS"),
-      FQ = summand2.give("FACETS");
+      FP = summands[0].give("FACETS"),
+      FQ = summands[1].give("FACETS");
 
-   Map<Vector<E>, int> index_of;
-   int n(0);
+   Map<Vector<E>, Int> index_of;
+   Int n = 0;
 
    // We will store the cells of the complex in a FacetList.
    // Initially, we reserve space for #vert Q vertices, 
@@ -115,17 +114,16 @@ perl::Object tiling_quotient(perl::Object P, perl::Object Q)
       translated_facets.col(0) -= FQ.minor(All, range_from(1)) * (*lit);
 
       // intersect the translated polytope with P
-      perl::Object Cell("Polytope", mlist<E>());
-      Cell.take("INEQUALITIES") << (translated_facets / FP);
+      BigObject Cell("Polytope", mlist<E>(), "INEQUALITIES", translated_facets / FP);
 
       // only proceed with full-dimensional faces
-      const int dd = Cell.give("COMBINATORIAL_DIM");
+      const Int dd = Cell.give("COMBINATORIAL_DIM");
       if (dd != d) continue;
 
       const Matrix<E> translated_vertices = Cell.give("VERTICES");
 
       // translate back, simultaneously store vertices and cell information
-      Set<int> vif;
+      Set<Int> vif;
       for (auto rit = entire(rows(translated_vertices)); !rit.at_end(); ++rit) {
          // translate back from the lattice point and undo the translation of the barycenter 
          const Vector<E> v (dehomogenize(*rit) - *lit + barycenter);
@@ -140,21 +138,20 @@ perl::Object tiling_quotient(perl::Object P, perl::Object Q)
    }
 
    // done
-   perl::Object PS("PolyhedralComplex", mlist<E>());
-   PS.take("VERTICES") << coos;
-   PS.take("MAXIMAL_POLYTOPES") << F;
-   return PS;
+   return BigObject("PolyhedralComplex", mlist<E>(),
+                    "VERTICES", coos,
+                    "MAXIMAL_POLYTOPES", F);
 }
 
 UserFunctionTemplate4perl("# @category Producing a polyhedral complex"
 			  "# Calculates the quotient of //P// by //Q//+L, where //Q//+L is a lattice tiling."
-           "# The result is a polytopal complex inside //Q//. "
+                          "# The result is a polytopal complex inside //Q//. "
 			  "# @param Polytope P a polytope"
 			  "# @param Polytope Q a polytope that tiles space"
 			  "# @tparam Coord"
 			  "# @return PolyhedralComplex",
 			  "tiling_quotient<E>(Polytope<E>, Polytope<E>)");
-    } }
+} }
 
 // Local Variables:
 // mode:C++
