@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -22,23 +23,22 @@
 namespace polymake { namespace polytope {
 
 template <typename Scalar>
-perl::Object facet_to_infinity(perl::Object p_in, const int inf_facet)
+BigObject facet_to_infinity(BigObject p_in, const Int inf_facet)
 {
+   BigObject p_out("Polytope", mlist<Scalar>());
 
-  perl::Object p_out(perl::ObjectType::construct<Scalar>("Polytope"));
+   const Matrix<Scalar> facets=p_in.give("FACETS");
 
-  const Matrix<Scalar> facets=p_in.give("FACETS");
-
-  const Vector<Scalar> inf_f= facets.row(inf_facet);
-  const Matrix<Scalar> tau=null_space(inf_f);
-  transform_section(p_out, p_in, "FACETS | INEQUALITIES", T(inf_f/tau));
-  transform_section(p_out, p_in, "AFFINE_HULL | EQUATIONS", T(inf_f/tau));
-  transform_section(p_out, p_in, "VERTICES", inv(inf_f/tau));
-  transform_section(p_out, p_in, "LINEALITY_SPACE", inv(inf_f/tau));
+   const Vector<Scalar> inf_f= facets.row(inf_facet);
+   const Matrix<Scalar> tau=null_space(inf_f);
+   transform_section(p_out, p_in, "FACETS | INEQUALITIES", T(inf_f/tau));
+   transform_section(p_out, p_in, "AFFINE_HULL | EQUATIONS", T(inf_f/tau));
+   transform_section(p_out, p_in, "VERTICES", inv(inf_f/tau));
+   transform_section(p_out, p_in, "LINEALITY_SPACE", inv(inf_f/tau));
   
-  p_out.take("BOUNDED") << false;
+   p_out.take("BOUNDED") << false;
 
-  return p_out;
+   return p_out;
 }
 
   UserFunctionTemplate4perl("# @category Producing a polytope from polytopes"
@@ -47,7 +47,7 @@ perl::Object facet_to_infinity(perl::Object p_in, const int inf_facet)
                             "# @param Int i the facet index"
                             "# @return Polytope"
                             "# @author Sven Herrmann"
-                            "# @example [prefer cdd] This generates the polytope that is the positive quadrant in 2-space:"
+                            "# @example [prefer cdd] [require bundled:cdd] This generates the polytope that is the positive quadrant in 2-space:"
                             "# > $q = new Polytope(VERTICES=>[[1,-1,-1],[1,0,1],[1,1,0]]);"
                             "# > $pf = facet_to_infinity($q,2);"
                             "# > print $pf->VERTICES;"
