@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -28,23 +28,20 @@ namespace polymake { namespace polytope {
 /*
  *  computes the integer hull of a polyhedron
  */
-perl::Object integer_hull(perl::Object p_in)
+BigObject integer_hull(BigObject p_in)
 {
   // compute the lattice points contained in the polyhedron
   // FIXME: use LATTICE_POINTS_GENERATORS instead
   Matrix<Rational> Lattice_Points = p_in.call_method("LATTICE_POINTS");
-  const int dim = p_in.give("CONE_AMBIENT_DIM");
-
-  perl::Object p_out("Polytope<Rational>");
+  const Int dim = p_in.give("CONE_AMBIENT_DIM");
 
   // define a new polyhedron as a convex hull of all the lattice points
-  p_out.take("POINTS") << Lattice_Points;
-  p_out.take("FEASIBLE") << (Lattice_Points.rows() > 0);
-  p_out.take("BOUNDED") << true;
-  p_out.take("POINTED") << true;
-  p_out.take("CONE_AMBIENT_DIM") << dim;
-
-  return p_out;	
+  return BigObject("Polytope<Rational>",
+                   "POINTS", Lattice_Points,
+                   "FEASIBLE", Lattice_Points.rows() > 0,
+                   "BOUNDED", true,
+                   "POINTED", true,
+                   "CONE_AMBIENT_DIM", dim);	
 }
 
 UserFunction4perl("# @category Producing a polytope from polytopes"
