@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,7 +26,7 @@
 namespace polymake { namespace polytope {
 
 template<typename Scalar>
-perl::Object pyramid(perl::Object p_in, const Scalar& z, perl::OptionSet options)
+BigObject pyramid(BigObject p_in, const Scalar& z, OptionSet options)
 {
    const bool noc = options["no_coordinates"],
       relabel = !options["no_labels"],
@@ -36,14 +37,14 @@ perl::Object pyramid(perl::Object p_in, const Scalar& z, perl::OptionSet options
    if (group && !p_in.exists("GROUP"))
       throw std::runtime_error("pyramid: group of the base polytope needs to be provided in order to compute group of the pyramid.");
 
-   int n_vertices=0;
-   perl::Object p_out("Polytope", mlist<Scalar>());
+   Int n_vertices = 0;
+   BigObject p_out("Polytope", mlist<Scalar>());
    p_out.set_description() << "pyramid over " << p_in.name() << endl;
 
    if (noc || p_in.exists("VERTICES_IN_FACETS")) {
-      const IncidenceMatrix<> VIF=p_in.give("VERTICES_IN_FACETS");
-      n_vertices=VIF.cols();
-      const int n_facets=VIF.rows();
+      const IncidenceMatrix<> VIF = p_in.give("VERTICES_IN_FACETS");
+      n_vertices = VIF.cols();
+      const Int n_facets = VIF.rows();
       const IncidenceMatrix<> VIF_out= (VIF | sequence(0,n_facets))     // original vertices + the new top vertex
          / sequence(0, n_vertices);     // original polytope becomes the bottom facet
 
@@ -52,7 +53,7 @@ perl::Object pyramid(perl::Object p_in, const Scalar& z, perl::OptionSet options
    }
    if (noc) {
       if (p_in.exists("COMBINATORIAL_DIM")) {
-         const int dim=p_in.give("COMBINATORIAL_DIM");
+         const Int dim = p_in.give("COMBINATORIAL_DIM");
          p_out.take("COMBINATORIAL_DIM") << dim+1;
       }
    } else {
@@ -67,18 +68,15 @@ perl::Object pyramid(perl::Object p_in, const Scalar& z, perl::OptionSet options
          (z0 | z);
    }
 
-   if(group){
-      Array<Array<int>> gens = p_in.give("GROUP.VERTICES_ACTION.GENERATORS");
+   if (group) {
+      Array<Array<Int>> gens = p_in.give("GROUP.VERTICES_ACTION.GENERATORS");
 
-      for(auto i = entire(gens); !i.at_end(); ++i)
+      for (auto i = entire(gens); !i.at_end(); ++i)
          (*i).resize(n_vertices+1,n_vertices);
 
-      perl::Object a("group::PermutationAction");
-      a.take("GENERATORS") << gens;
-
-      perl::Object g("group::Group");
+      BigObject a("group::PermutationAction", "GENERATORS", gens);
+      BigObject g("group::Group", "canonicalGroup");
       g.set_description() << "canonical group induced by the group of the base polytope" << endl;
-      g.set_name("canonicalGroup");
       p_out.take("GROUP") << g;
       p_out.take("GROUP.VERTICES_ACTION") << a;
    }
