@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -25,11 +25,11 @@
 
 namespace polymake { namespace polytope {
 
-Array<RGB> vertex_colors(perl::Object p, perl::Object lp, perl::OptionSet options)
+Array<RGB> vertex_colors(BigObject p, BigObject lp, OptionSet options)
 {
-   const Set<int> max_face=lp.give("MAXIMAL_FACE"),
-      min_face=lp.give("MINIMAL_FACE");
-   Set<int> rays;
+   const Set<Int> max_face = lp.give("MAXIMAL_FACE"),
+                  min_face = lp.give("MINIMAL_FACE");
+   Set<Int> rays;
    Vector<double> obj;
 
    if (lp.lookup("LINEAR_OBJECTIVE", allow_conversion()) >> obj) {
@@ -47,11 +47,11 @@ Array<RGB> vertex_colors(perl::Object p, perl::Object lp, perl::OptionSet option
 
    if (!upper_bound || !lower_bound) {
       const Graph<> G=p.give("GRAPH.ADJACENCY");
-      const Set<int> neighbors_of_far=accumulate(select(rows(adjacency_matrix(G)),rays), operations::add())-rays;
+      const Set<Int> neighbors_of_far = accumulate(select(rows(adjacency_matrix(G)),rays), operations::add())-rays;
       if (!upper_bound)
-         max_val=accumulate(obj.slice(neighbors_of_far), operations::max());
+         max_val = accumulate(obj.slice(neighbors_of_far), operations::max());
       if (!lower_bound)
-         min_val=accumulate(obj.slice(neighbors_of_far), operations::min());
+         min_val = accumulate(obj.slice(neighbors_of_far), operations::min());
    }
 
    double diff=max_val-min_val;
@@ -59,11 +59,11 @@ Array<RGB> vertex_colors(perl::Object p, perl::Object lp, perl::OptionSet option
    if (!lower_bound) min_val-=0.1*diff;
    diff=max_val-min_val;
 
-   const int n_vertices=obj.size();
+   const Int n_vertices = obj.size();
    const RGB white(1.,1.,1.);
    Array<RGB> VC(n_vertices, white);
 
-   if (diff) {
+   if (diff != 0) {
       RGB max_rgb(white), min_rgb;
       options["max"] >> max_rgb;
       options["min"] >> min_rgb;
@@ -71,29 +71,29 @@ Array<RGB> vertex_colors(perl::Object p, perl::Object lp, perl::OptionSet option
 
       // If the maximal (minimal) face is affine, it is fully saturated,
       // otherwise the points towards the unbounded direction are made pale.
-      if (!upper_bound) maxcolor.saturation*=0.5;
-      if (!lower_bound) mincolor.saturation*=0.5;
+      if (!upper_bound) maxcolor.saturation *= 0.5;
+      if (!lower_bound) mincolor.saturation *= 0.5;
 
-      const double Hmin=mincolor.hue,
-         Hmax=maxcolor.hue,
-         Hdiff=Hmax-Hmin,
-         Smin=mincolor.saturation,
-         Smax=maxcolor.saturation,
-         Sdiff=Smax-Smin,
-         Vmin=mincolor.value,
-         Vmax=maxcolor.value,
-         Vdiff=Vmax-Vmin;
+      const double Hmin = mincolor.hue,
+                   Hmax = maxcolor.hue,
+                  Hdiff = Hmax-Hmin,
+                   Smin = mincolor.saturation,
+                   Smax = maxcolor.saturation,
+                  Sdiff = Smax-Smin,
+                   Vmin = mincolor.value,
+                   Vmax = maxcolor.value,
+                  Vdiff = Vmax-Vmin;
 
-      Array<RGB>::iterator c=VC.begin();
-      for (int v=0; v<n_vertices; ++v, ++c) {
+      auto c = VC.begin();
+      for (Int v = 0; v < n_vertices; ++v, ++c) {
          if (rays.contains(v)) {
             if (max_face.contains(v))
-               *c=maxcolor;
+               *c = maxcolor;
             else if (min_face.contains(v))
-               *c=mincolor;
+               *c = mincolor;
          } else {
-            const double k=(max_val-obj[v])/diff;
-            *c=HSV(Hmax-k*Hdiff, Smax-k*Sdiff, Vmax-k*Vdiff);
+            const double k = (max_val-obj[v])/diff;
+            *c = HSV(Hmax-k*Hdiff, Smax-k*Sdiff, Vmax-k*Vdiff);
          }
       }
    }
