@@ -31,111 +31,145 @@ multi_associahedron_sphere(Int n, Int k, OptionSet options)
       nu (n-2*k-1),       // the complexity measure
       dim(k*nu-1);        // the dimension of the sphere
 
-   const bool no_facets    = options["no_facets"];
-   const bool no_crossings = options["no_crossings"];
+   const bool include_facets    = options["include_facets"];
+   const Array<Set<Int>> link_of_diagonals = options["link_of_diagonals"];
+   const bool include_crossings = link_of_diagonals.size()
+      ? false
+      : options["include_crossings"];
    
+   if (n<=2) throw std::runtime_error("multi_associahedron_sphere: require n>=3");
    if (k<=0) throw std::runtime_error("multi_associahedron_sphere: require k>=1");
    if (m<=0) throw std::runtime_error("multi_associahedron_sphere: require (n choose 2) > n*k");
-   assert(n>=3);
 
    DiagonalIndex index_of;
    DiagonalList diagonals; diagonals.reserve(m);
    DiagonalLabels labels; labels.reserve(m);
    prepare_diagonal_data(n, k, index_of, diagonals, labels);
    assert(index_of.size() == size_t(m));
+
+   Set<Int> link_of;
+   for (const auto& d: link_of_diagonals) {
+      if (2 != d.size())
+         throw std::runtime_error("multi_associahedron_sphere: diagonals in option link_of_diagonals must have size 2");
+      const auto key(std::make_pair(d.front(), d.back()));
+      if (!index_of.exists(key))
+         throw std::runtime_error("multi_associahedron_sphere: diagonal given in link_of_diagonals is not actually a diagonal");
+      link_of += index_of[key];
+   }
    
    BigObject ind_Aut("group::Group");
    BigObject induced_action("group::PermutationAction");
    Array<Array<Int>> igens;
-   
-   if (nu > 1) {
-      BigObject Aut = group::dihedral_group(2*n);
-      const Array<Array<Int>> gens = Aut.give("PERMUTATION_ACTION.GENERATORS");
-      igens = induced_action_gens_impl(gens, diagonals, index_of);
-   
-      induced_action.set_description("action of D_" + std::to_string(2*n)
-                                     + " on the vertices of the simplicial complex, induced by the action of D_"
-                                     + std::to_string(2*n) + " on the vertices of the polygon");
-   
-      const group::ConjugacyClassReps<> class_reps = Aut.give("PERMUTATION_ACTION.CONJUGACY_CLASS_REPRESENTATIVES");
-      group::ConjugacyClassReps<> induced_ccr(class_reps.size());
-      auto iicr_it = entire(induced_ccr);
-      for (const auto& r: class_reps) {
-         *iicr_it = induced_gen(r, diagonals, index_of);
-         ++iicr_it;
-      }
-      induced_action.take("CONJUGACY_CLASS_REPRESENTATIVES") << induced_ccr;
-      ind_Aut.take("CHARACTER_TABLE") << group::dn_character_table(2*n);
-      ind_Aut.take("ORDER") << 2*n;
-   } else {
-      induced_action.set_description("action of S" + std::to_string(m)
-                                     + " on the vertices of the simplicial complex, induced by the action of D_"
-                                     + std::to_string(2*n) + " on the vertices of the polygon");
-      igens = group::symmetric_group_gens(m);
-      if (m<8) {
-         induced_action.take("CONJUGACY_CLASS_REPRESENTATIVES") << group::sn_reps(m);
-         ind_Aut.take("CHARACTER_TABLE") << group::sn_character_table(m);
-      }
-      ind_Aut.take("ORDER") << Integer::fac(m);
-   }
 
-   induced_action.take("GENERATORS") << igens;
-   ind_Aut.take("PERMUTATION_ACTION") << induced_action;
+   if (!link_of.size()) {
+      if (nu > 1) {
+         dihedral_group_action(n, diagonals, index_of, ind_Aut, induced_action, igens);
+      } else {
+         symmetric_group_action(m, n, index_of, ind_Aut, induced_action, igens);
+      }
+   
+      induced_action.take("GENERATORS") << igens;
+      ind_Aut.take("PERMUTATION_ACTION") << induced_action;
+   }
       
-   hash_set<Set<Int>> lower_rep_level, k_plus_1_crossings;
+   hash_set<Set<Int>> lower_layer, k_plus_1_crossings;
 
-   if (!no_crossings || !no_facets) {
+   /*
+     We build up the faces of the simplicial complex dimension by dimension by
+     - maintaining a lower and an upper layer in the Hasse diagram,
+     - initializing the lower layer to be set of all //k//-crossings,
+     - deducing the next upper layer by
+        -- for each face F in the lower layer
+        -- add each possible new vertex index to it
+        -- if that set contains no //k+1//-crossing, add it to the next layer
+     - finally, setting lower_layer = upper_layer
+     - and repeating until the number of vertices prescribed by the dimension
+       is reached.
+
+     There is another complication in that we sometimes want to calculate
+     just the link of a face F = link_of, because the sphere might get too big otherwise.
+     
+     If that is the case, we seed lower_layer 
+     - not with all elements of [n] choose k,
+     - but only with the set F,
+     and at the very end, we remove F from all faces.
+    */
+
+   Int current_simplex_size(-1);
+   
+   // initialize the lower_layer
+   if (include_crossings || include_facets) {
       // accumulate all k-cliques, to prepare for calculating the (k+1)-crossings and the f-vector
-      for (auto cit = entire(all_subsets_of_k(sequence(0,m), k)); !cit.at_end(); ++cit)
-         lower_rep_level += *cit;
-   }
-
-   if (!no_crossings) {
-      // now calculate the (k+1)-crossings
-      for (const auto& c : lower_rep_level)
-         for (Int d_index = 0; d_index < m; ++d_index)
-            if (!c.contains(d_index) && contains_new_k_plus_1_crossing(d_index, k, c, diagonals))
-               k_plus_1_crossings += group::unordered_orbit<group::on_container>(igens, Set<Int>(c+d_index));
+      if (link_of.size()) {
+         lower_layer += link_of;
+         current_simplex_size = link_of.size();
+      } else {
+         for (auto cit = entire(all_subsets_of_k(sequence(0,m), k)); !cit.at_end(); ++cit) 
+            lower_layer += *cit;
+         current_simplex_size = k;
+      }
    }
    
-   Array<Int> f_vector(dim+1);
+   if (include_crossings) 
+      // now calculate the (k+1)-crossings
+      fill_upper_layer(k_plus_1_crossings, lower_layer, k, m, diagonals, igens, link_of, IncludeCrossings::yes);
 
-   if (!no_facets) {
-      // the complex is k-neighborly, so we know part of the f-vector
-      auto fvec_it = f_vector.begin();
-      *fvec_it++ = m;
-      for (Int kk = 2; kk <= k; ++kk)
-         *fvec_it++ = Int(Integer::binom(m,kk));
+   Array<Int> f_vector(dim+1);
+   auto fvec_it = f_vector.begin();
+   
+   if (include_facets) {
+      if (!link_of.size())
+         initialize_f_vector(fvec_it, m, k);
 
       // build up the face lattice in layers, starting from representatives of the k-cliques
-      for (Int n_vertices = k+1; n_vertices <= dim+1; ++n_vertices) {
-         hash_set<Set<Int>> upper_rep_level;
-         for (const auto& c : lower_rep_level)
-            for (Int d_index = 0; d_index < m; ++d_index)
-               if (!c.contains(d_index) && !contains_new_k_plus_1_crossing(d_index, k, c, diagonals))
-                  upper_rep_level += group::unordered_orbit<group::on_container>(igens, Set<Int>(c+d_index));
+      for (++current_simplex_size; current_simplex_size <= dim+1; ++current_simplex_size) {
+         hash_set<Set<Int>> upper_layer;
+         fill_upper_layer(upper_layer, lower_layer, k, m, diagonals, igens, link_of, IncludeCrossings::no);
+         
+         if (!link_of.size())
+            *fvec_it++ = upper_layer.size();
 
-         *fvec_it++ = upper_rep_level.size();
-         lower_rep_level = upper_rep_level;
+         lower_layer = upper_layer;
       }
    }
+
+   IncidenceMatrix<> facets;
+   if (!link_of.size())
+      facets = IncidenceMatrix<>(lower_layer);
+   else 
+      squeeze_matrix(facets, labels, lower_layer, link_of);
    
    BigObject s("SimplicialComplex");
-   s.set_description() << "Simplicial complex of " << k << "-triangulations of a convex " << n << "-gon" << endl;
-   s.take("N_VERTICES") << m;
+   std::ostringstream os;
+   if (link_of.size()) {
+      wrap(os) << "Link of face "
+               << link_of
+               << " = ";
+      for (const auto& d: link_of_diagonals)
+         wrap(os) << d << " ";
+      os << "of ";
+   }
+   os << "Simplicial complex of " << k << "-triangulations of a convex " << n << "-gon";
+   s.set_description() << os.str() << endl;
+   s.take("N_VERTICES") << labels.size();
    s.take("VERTEX_LABELS") << labels;
-   s.take("DIM") << dim;
+   s.take("DIM") << dim - link_of.size();
    s.take("PURE") << true;
    s.take("SPHERE") << true;
-   s.take("GROUP") << ind_Aut;
 
-   if (!no_crossings) {
+   if (!link_of_diagonals.size())
+      s.take("GROUP") << ind_Aut;
+
+   if (include_crossings) {
       s.attach("K_PLUS_1_CROSSINGS") << Set<Set<Int>>(entire(k_plus_1_crossings));
    }
    
-   if (!no_facets) {
-      s.take("FACETS") << Set<Set<Int>>(entire(lower_rep_level));
-      s.take("F_VECTOR") << f_vector;
+   if (include_facets) {
+      const Set<Set<Int>> facet_set(entire(rows(facets)));
+      s.take("N_FACETS") << facet_set.size();
+      s.take("FACETS") << facet_set;
+      if (!link_of_diagonals.size())
+         s.take("F_VECTOR") << f_vector;
    }
    
    return s;
@@ -160,14 +194,16 @@ UserFunction4perl("# @category Producing from scratch"
                   "# also be obtained as the property MINIMAL_NON_FACES, but this requires the HASSE_DIAGRAM to be computed."
                   "# @param Int n the number of vertices of the polygon"
                   "# @param Int k the number of diagonals that are allowed to mutually cross"
-                  "# @option Bool no_facets don't calculate the facets (for large examples)? Default 0"
-                  "# @option Bool no_crossings don't calculate the crossings? Default 0"
+                  "# @option Bool include_facets calculate the facets (for large examples)? Default 1"
+                  "# @option Bool include_crossings calculate the crossings? Default 1"
+                  "# @option Array<Set<Int>> link_of_diagonals calculate the link of the sphere of the given diagonals."
+                  "# This option implies include_crossings=>0 and causes no GROUP to be generated"
                   "# @return SimplicialComplex"
                   "# @example The f-vector of Δ(9,3) is that of a neighborly polytope, since ν=2:"
                   "# > print multi_associahedron_sphere(9,3)->F_VECTOR;"
                   "# | 9 36 84 117 90 30"
-                  "# @example The option no_facets=>1 still leaves useful information:"
-                  "# > $s = multi_associahedron_sphere(8,2, no_facets=>1);"
+                  "# @example The option include_facets=>0 still leaves useful information:"
+                  "# > $s = multi_associahedron_sphere(8,2, include_facets=>0);"
                   "# > print $s->VERTEX_LABELS;"
                   "# | (0 3) (1 4) (2 5) (3 6) (4 7) (0 5) (1 6) (2 7) (0 4) (1 5) (2 6) (3 7)"
                   "# > print $s->GROUP->PERMUTATION_ACTION->GENERATORS;"
@@ -177,7 +213,7 @@ UserFunction4perl("# @category Producing from scratch"
                   "# | 28"
                   ,
                   &multi_associahedron_sphere,
-                  "multi_associahedron_sphere($$ { no_facets=>0, no_crossings=>0 })");
+                  "multi_associahedron_sphere($$ { include_facets=>1, include_crossings=>1, link_of_diagonals=>[] })");
 
 } }
 
